@@ -94,6 +94,8 @@ class DarwinHardware(Hardware):
 
         total_used = 0
         page_size = 4096
+        if 'hw.pagesize' in self.sysctl:
+            page_size = int(self.sysctl['hw.pagesize'])
 
         vm_stat_command = self.module.get_bin_path('vm_stat')
         if vm_stat_command is None:
