@@ -15,15 +15,13 @@
 # You should have received a copy of the GNU General Public License
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 import os
 
 from ansible import constants as C
-from ansible.errors import AnsibleParserError, AnsibleUndefinedVariable, AnsibleFileNotFound, AnsibleAssertionError
-from ansible.module_utils._text import to_native
-from ansible.module_utils.six import string_types
+from ansible.errors import AnsibleParserError, AnsibleUndefinedVariable, AnsibleAssertionError
+from ansible.module_utils.common.text.converters import to_native
 from ansible.parsing.mod_args import ModuleArgsParser
 from ansible.utils.display import Display
 
@@ -105,6 +103,8 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
             raise AnsibleAssertionError('The ds (%s) should be a dict but was a %s' % (ds, type(ds)))
 
         if 'block' in task_ds:
+            if use_handlers:
+                raise AnsibleParserError("Using a block as a handler is not supported.", obj=task_ds)
             t = Block.load(
                 task_ds,
                 play=play,
@@ -148,28 +148,9 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                 templar = Templar(loader=loader, variables=all_vars)
 
                 # check to see if this include is dynamic or static:
-                # 1. the user has set the 'static' option to false or true
-                # 2. one of the appropriate config options was set
-                if action in C._ACTION_INCLUDE_TASKS:
-                    is_static = False
-                elif action in C._ACTION_IMPORT_TASKS:
-                    is_static = True
-                elif t.static is not None:
-                    display.deprecated("The use of 'static' has been deprecated. "
-                                       "Use 'import_tasks' for static inclusion, or 'include_tasks' for dynamic inclusion",
-                                       version='2.12', collection_name='ansible.builtin')
-                    is_static = t.static
-                else:
-                    is_static = C.DEFAULT_TASK_INCLUDES_STATIC or \
-                        (use_handlers and C.DEFAULT_HANDLER_INCLUDES_STATIC) or \
-                        (not templar.is_template(t.args['_raw_params']) and t.all_parents_static() and not t.loop)
-
-                if is_static:
+                if action in C._ACTION_IMPORT_TASKS:
                     if t.loop is not None:
-                        if action in C._ACTION_IMPORT_TASKS:
-                            raise AnsibleParserError("You cannot use loops on 'import_tasks' statements. You should use 'include_tasks' instead.", obj=task_ds)
-                        else:
-                            raise AnsibleParserError("You cannot use 'static' on an include with a loop", obj=task_ds)
+                        raise AnsibleParserError("You cannot use loops on 'import_tasks' statements. You should use 'include_tasks' instead.", obj=task_ds)
 
                     # we set a flag to indicate this include was static
                     t.statically_loaded = True
@@ -234,35 +215,18 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                         else:
                             include_file = loader.path_dwim(include_target)
 
-                    try:
-                        data = loader.load_from_file(include_file)
-                        if not data:
-                            display.warning('file %s is empty and had no tasks to include' % include_file)
-                            continue
-                        elif not isinstance(data, list):
-                            raise AnsibleParserError("included task files must contain a list of tasks", obj=data)
-
-                        # since we can't send callbacks here, we display a message directly in
-                        # the same fashion used by the on_include callback. We also do it here,
-                        # because the recursive nature of helper methods means we may be loading
-                        # nested includes, and we want the include order printed correctly
-                        display.vv("statically imported: %s" % include_file)
-                    except AnsibleFileNotFound:
-                        if action not in C._ACTION_INCLUDE or t.static or \
-                           C.DEFAULT_TASK_INCLUDES_STATIC or \
-                           C.DEFAULT_HANDLER_INCLUDES_STATIC and use_handlers:
-                            raise
-                        display.deprecated(
-                            "Included file '%s' not found, however since this include is not "
-                            "explicitly marked as 'static: yes', we will try and include it dynamically "
-                            "later. In the future, this will be an error unless 'static: no' is used "
-                            "on the include task. If you do not want missing includes to be considered "
-                            "dynamic, use 'static: yes' on the include or set the global ansible.cfg "
-                            "options to make all includes static for tasks and/or handlers" % include_file,
-                            version="2.12", collection_name='ansible.builtin'
-                        )
-                        task_list.append(t)
+                    data = loader.load_from_file(include_file)
+                    if not data:
+                        display.warning('file %s is empty and had no tasks to include' % include_file)
                         continue
+                    elif not isinstance(data, list):
+                        raise AnsibleParserError("included task files must contain a list of tasks", obj=data)
+
+                    # since we can't send callbacks here, we display a message directly in
+                    # the same fashion used by the on_include callback. We also do it here,
+                    # because the recursive nature of helper methods means we may be loading
+                    # nested includes, and we want the include order printed correctly
+                    display.vv("statically imported: %s" % include_file)
 
                     ti_copy = t.copy(exclude_parent=True)
                     ti_copy._parent = block
@@ -277,28 +241,7 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                         variable_manager=variable_manager,
                     )
 
-                    # FIXME: remove once 'include' is removed
-                    # pop tags out of the include args, if they were specified there, and assign
-                    # them to the include. If the include already had tags specified, we raise an
-                    # error so that users know not to specify them both ways
-                    tags = ti_copy.vars.pop('tags', [])
-                    if isinstance(tags, string_types):
-                        tags = tags.split(',')
-
-                    if len(tags) > 0:
-                        if action in C._ACTION_ALL_PROPER_INCLUDE_IMPORT_TASKS:
-                            raise AnsibleParserError('You cannot specify "tags" inline to the task, it is a task keyword')
-                        if len(ti_copy.tags) > 0:
-                            raise AnsibleParserError(
-                                "Include tasks should not specify tags in more than one way (both via args and directly on the task). "
-                                "Mixing styles in which tags are specified is prohibited for whole import hierarchy, not only for single import statement",
-                                obj=task_ds,
-                                suppress_extended_error=True,
-                            )
-                        display.deprecated("You should not specify tags in the include parameters. All tags should be specified using the task-level option",
-                                           version="2.12", collection_name='ansible.builtin')
-                    else:
-                        tags = ti_copy.tags[:]
+                    tags = ti_copy.tags[:]
 
                     # now we extend the tags on each of the included blocks
                     for b in included_blocks:
@@ -313,10 +256,12 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                     else:
                         task_list.extend(included_blocks)
                 else:
-                    t.is_static = False
                     task_list.append(t)
 
             elif action in C._ACTION_ALL_PROPER_INCLUDE_IMPORT_ROLES:
+                if use_handlers:
+                    raise AnsibleParserError(f"Using '{action}' as a handler is not supported.", obj=task_ds)
+
                 ir = IncludeRole.load(
                     task_ds,
                     block=block,
@@ -326,24 +271,9 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                     loader=loader,
                 )
 
-                #   1. the user has set the 'static' option to false or true
-                #   2. one of the appropriate config options was set
-                is_static = False
                 if action in C._ACTION_IMPORT_ROLE:
-                    is_static = True
-
-                elif ir.static is not None:
-                    display.deprecated("The use of 'static' for 'include_role' has been deprecated. "
-                                       "Use 'import_role' for static inclusion, or 'include_role' for dynamic inclusion",
-                                       version='2.12', collection_name='ansible.builtin')
-                    is_static = ir.static
-
-                if is_static:
                     if ir.loop is not None:
-                        if action in C._ACTION_IMPORT_ROLE:
-                            raise AnsibleParserError("You cannot use loops on 'import_role' statements. You should use 'include_role' instead.", obj=task_ds)
-                        else:
-                            raise AnsibleParserError("You cannot use 'static' on an include_role with a loop", obj=task_ds)
+                        raise AnsibleParserError("You cannot use loops on 'import_role' statements. You should use 'include_role' instead.", obj=task_ds)
 
                     # we set a flag to indicate this include was static
                     ir.statically_loaded = True
@@ -351,10 +281,11 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
                     # template the role name now, if needed
                     all_vars = variable_manager.get_vars(play=play, task=ir)
                     templar = Templar(loader=loader, variables=all_vars)
+                    ir.post_validate(templar=templar)
                     ir._role_name = templar.template(ir._role_name)
 
                     # uses compiled list from object
-                    blocks, _ = ir.get_block_list(variable_manager=variable_manager, loader=loader)
+                    blocks, dummy = ir.get_block_list(variable_manager=variable_manager, loader=loader)
                     task_list.extend(blocks)
                 else:
                     # passes task object itself for latter generation of list
@@ -362,8 +293,12 @@ def load_list_of_tasks(ds, play, block=None, role=None, task_include=None, use_h
             else:
                 if use_handlers:
                     t = Handler.load(task_ds, block=block, role=role, task_include=task_include, variable_manager=variable_manager, loader=loader)
+                    if t.action in C._ACTION_META and t.args.get('_raw_params') == "end_role":
+                        raise AnsibleParserError("Cannot execute 'end_role' from a handler")
                 else:
                     t = Task.load(task_ds, block=block, role=role, task_include=task_include, variable_manager=variable_manager, loader=loader)
+                    if t.action in C._ACTION_META and t.args.get('_raw_params') == "end_role" and role is None:
+                        raise AnsibleParserError("Cannot execute 'end_role' from outside of a role")
 
                 task_list.append(t)
 
