@@ -834,8 +834,6 @@ class PathMapper:
                 'MANIFEST.in',
                 'pyproject.toml',
                 'requirements.txt',
-                'setup.cfg',
-                'setup.py',
             ):
                 return packaging
 
