@@ -8,7 +8,7 @@ from ..data import (
 )
 
 
-def resolve_csharp_ps_util(import_name, path):  # type: (str, str) -> str
+def resolve_csharp_ps_util(import_name: str, path: str) -> str:
     """Return the fully qualified name of the given import if possible, otherwise return the original import name."""
     if data_context().content.is_ansible or not import_name.startswith('.'):
         # We don't support relative paths for builtin utils, there's no point.
