@@ -84,24 +84,31 @@ def get_option_name(name: str) -> str:
 
 class PythonVersionUnsupportedError(ApplicationError):
     """A Python version was requested for a context which does not support that version."""
+
     def __init__(self, context: str, version: str, versions: c.Iterable[str]) -> None:
         super().__init__(f'Python {version} is not supported by environment `{context}`. Supported Python version(s) are: {", ".join(versions)}')
 
 
 class PythonVersionUnspecifiedError(ApplicationError):
     """A Python version was not specified for a context which is unknown, thus the Python version is unknown."""
+
     def __init__(self, context: str) -> None:
-        super().__init__(f'A Python version was not specified for environment `{context}`. Use the `--python` option to specify a Python version.')
+        super().__init__(
+            f'Environment `{context}` is unknown. Use a predefined environment instead. '
+            f'Alternatively, to use an unknown environment, use the `--python` option to specify a Python version.'
+        )
 
 
 class ControllerNotSupportedError(ApplicationError):
     """Option(s) were specified which do not provide support for the controller and would be ignored because they are irrelevant for the target."""
+
     def __init__(self, context: str) -> None:
         super().__init__(f'Environment `{context}` does not provide a Python version supported by the controller.')
 
 
 class OptionsConflictError(ApplicationError):
     """Option(s) were specified which conflict with other options."""
+
     def __init__(self, first: c.Iterable[str], second: c.Iterable[str]) -> None:
         super().__init__(f'Options `{" ".join(first)}` cannot be combined with options `{" ".join(second)}`.')
 
@@ -109,6 +116,7 @@ class OptionsConflictError(ApplicationError):
 @dataclasses.dataclass(frozen=True)
 class LegacyHostOptions:
     """Legacy host options used prior to the availability of separate controller and target host configuration."""
+
     python: t.Optional[str] = None
     python_interpreter: t.Optional[str] = None
     local: t.Optional[bool] = None
@@ -161,6 +169,7 @@ class LegacyHostOptions:
 
 class TargetMode(enum.Enum):
     """Type of provisioning to use for the targets."""
+
     WINDOWS_INTEGRATION = enum.auto()  # windows-integration
     NETWORK_INTEGRATION = enum.auto()  # network-integration
     POSIX_INTEGRATION = enum.auto()  # integration
