@@ -1,16 +1,14 @@
 """Utility code for facilitating collection of code coverage when running tests."""
 from __future__ import annotations
 
-import atexit
 import dataclasses
 import os
 import sqlite3
 import tempfile
+import textwrap
 import typing as t
 
 from .config import (
-    IntegrationConfig,
-    SanityConfig,
     TestConfig,
 )
 
@@ -34,6 +32,7 @@ from .data import (
 )
 
 from .util_common import (
+    ExitHandler,
     intercept_python,
     ResultType,
 )
@@ -60,6 +59,7 @@ from .thread import (
 @dataclasses.dataclass(frozen=True)
 class CoverageVersion:
     """Details about a coverage version and its supported Python versions."""
+
     coverage_version: str
     schema_version: int
     min_python: tuple[int, int]
@@ -68,8 +68,7 @@ class CoverageVersion:
 
 COVERAGE_VERSIONS = (
     # IMPORTANT: Keep this in sync with the ansible-test.txt requirements file.
-    CoverageVersion('6.5.0', 7, (3, 7), (3, 11)),
-    CoverageVersion('4.5.4', 0, (2, 6), (3, 6)),
+    CoverageVersion('7.6.1', 7, (3, 8), (3, 13)),
 )
 """
 This tuple specifies the coverage version to use for Python version ranges.
@@ -81,6 +80,7 @@ CONTROLLER_COVERAGE_VERSION = COVERAGE_VERSIONS[0]
 
 class CoverageError(ApplicationError):
     """Exception caused while attempting to read a coverage file."""
+
     def __init__(self, path: str, message: str) -> None:
         self.path = path
         self.message = message
@@ -215,13 +215,13 @@ def get_coverage_config(args: TestConfig) -> str:
     except AttributeError:
         pass
 
-    coverage_config = generate_coverage_config(args)
+    coverage_config = generate_coverage_config()
 
     if args.explain:
         temp_dir = '/tmp/coverage-temp-dir'
     else:
         temp_dir = tempfile.mkdtemp()
-        atexit.register(lambda: remove_tree(temp_dir))
+        ExitHandler.register(lambda: remove_tree(temp_dir))
 
     path = os.path.join(temp_dir, COVERAGE_CONFIG_NAME)
 
@@ -233,10 +233,10 @@ def get_coverage_config(args: TestConfig) -> str:
     return path
 
 
-def generate_coverage_config(args: TestConfig) -> str:
+def generate_coverage_config() -> str:
     """Generate code coverage configuration for tests."""
     if data_context().content.collection:
-        coverage_config = generate_collection_coverage_config(args)
+        coverage_config = generate_collection_coverage_config()
     else:
         coverage_config = generate_ansible_coverage_config()
 
@@ -248,7 +248,9 @@ def generate_ansible_coverage_config() -> str:
     coverage_config = '''
 [run]
 branch = True
-concurrency = multiprocessing
+concurrency =
+    multiprocessing
+    thread
 parallel = True
 
 omit =
@@ -261,40 +263,46 @@ omit =
     */test/results/*
 '''
 
+    coverage_config = coverage_config.lstrip()
+
     return coverage_config
 
 
-def generate_collection_coverage_config(args: TestConfig) -> str:
+def generate_collection_coverage_config() -> str:
     """Generate code coverage configuration for Ansible Collection tests."""
-    coverage_config = '''
+    include_patterns = [
+        # {base}/ansible_collections/{ns}/{col}/*
+        os.path.join(data_context().content.root, '*'),
+        # */ansible_collections/{ns}/{col}/* (required to pick up AnsiballZ coverage)
+        os.path.join('*', data_context().content.collection.directory, '*'),
+    ]
+
+    omit_patterns = [
+        # {base}/ansible_collections/{ns}/{col}/tests/output/*
+        os.path.join(data_context().content.root, data_context().content.results_path, '*'),
+    ]
+
+    include = textwrap.indent('\n'.join(include_patterns), ' ' * 4)
+    omit = textwrap.indent('\n'.join(omit_patterns), ' ' * 4)
+
+    coverage_config = f"""
 [run]
 branch = True
-concurrency = multiprocessing
+concurrency =
+    multiprocessing
+    thread
 parallel = True
 disable_warnings =
     no-data-collected
-'''
 
-    if isinstance(args, IntegrationConfig):
-        coverage_config += '''
 include =
-    %s/*
-    */%s/*
-''' % (data_context().content.root, data_context().content.collection.directory)
-    elif isinstance(args, SanityConfig):
-        # temporary work-around for import sanity test
-        coverage_config += '''
-include =
-    %s/*
+{include}
 
 omit =
-    %s/*
-''' % (data_context().content.root, os.path.join(data_context().content.root, data_context().content.results_path))
-    else:
-        coverage_config += '''
-include =
-     %s/*
-''' % data_context().content.root
+{omit}
+"""
+
+    coverage_config = coverage_config.lstrip()
 
     return coverage_config
 
